package	jp.co.sds_corp.util.jmx.rmi.server;

import java.rmi.RemoteException;
import java.rmi.ServerError;
import java.rmi.ServerException;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.Iterator;

import javax.management.Attribute;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanServer;
import javax.management.ObjectInstance;
import javax.management.ObjectName;
import javax.management.QueryExp;
import javax.management.ReflectionException;
import javax.management.RuntimeErrorException;
import javax.management.RuntimeMBeanException;
import javax.management.RuntimeOperationsException;

import jp.co.sds_corp.util.jmx.rmi.JMXAdaptor;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class JMXAdaptorImpl extends UnicastRemoteObject implements JMXAdaptor
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	MBeanServer server;

	public JMXAdaptorImpl(MBeanServer server) throws RemoteException {
		super();
		this.server = server;
	}

	// JMXAdaptor implementation -------------------------------------
	public java.lang.Object invoke( ObjectName name, String actionName,
		Object[] params, String[] signature )
		throws InstanceNotFoundException, MBeanException,
				ReflectionException, RemoteException
	{
		Object value = null;
		Exception ex = null;
		try {
			value = server.invoke(name, actionName, params, signature);
		} catch(MBeanException e) {
			ex = e.getTargetException();
		} catch(RuntimeErrorException e) {
			Error err = e.getTargetError();
			ex = new ServerError("RuntimeErrorException", err);
		} catch(RuntimeMBeanException e) {
			ex = e.getTargetException();
		} catch(RuntimeOperationsException e) {
			ex = e.getTargetException();
		}

		if( ex != null ) {
			String msg = "invoke name="+name+", action="+actionName+" failed";
			logger.debug( this, msg, ex);
			throw new ServerException(msg, ex);
		}

		return value;
	}

	public java.util.Set<ObjectInstance> queryMBeans(ObjectName name, QueryExp query)
		throws RemoteException
	{
		return server.queryMBeans(name, query);
	}

	public java.util.Collection<MBeanInfo> getMBeanInfos()
		throws RemoteException
	{
		try {
			ArrayList<MBeanInfo> infos = new ArrayList<MBeanInfo>();
			Iterator mbeans = server.queryNames(null, null).iterator();
			while(mbeans.hasNext()) {
				infos.add(server.getMBeanInfo((ObjectName)mbeans.next()));
			}
			return infos;
		} catch (Exception e) {
			throw new ServerException("Exception occurred", e);
		}
	}

	public void setAttribute(ObjectName name, Attribute attribute)
		throws InstanceNotFoundException, AttributeNotFoundException,
				InvalidAttributeValueException, MBeanException,
				ReflectionException, RemoteException
	{
		server.setAttribute(name, attribute);
	}

	public java.lang.Object getAttribute( ObjectName name, String attribute)
		throws MBeanException, AttributeNotFoundException,
				InstanceNotFoundException, ReflectionException,
				RemoteException
	{
		return server.getAttribute(name, attribute);
	}

}

